use std::{fs::File, io::Read, sync::Arc};

use poem::{
    error::NotFoundError,
    handler,
    web::{headers::ContentType, Data, Path, WithContentType},
    IntoResponse,
};

use crate::{app_state::AppState, handlers::error::HandlerError};

#[handler]
pub async fn handler(
    id: Path<u64>,
    state: Data<&Arc<AppState>>,
) -> Result<WithContentType<Vec<u8>>, poem::Error> {
    let book = state
        .calibre
        .scalar_book(*id)
        .map_err(HandlerError::DataError)?;
    let cover_path = state.config.library_path.join(book.path).join("cover.jpg");
    let mut cover = File::open(cover_path).map_err(|_| NotFoundError)?;

    let mut data = Vec::new();
    cover.read_to_end(&mut data).map_err(|_| NotFoundError)?;

    Ok(data.with_content_type(ContentType::jpeg().to_string()))
}
