use std::{fs::File, io::Read, sync::Arc};

use poem::{
    error::NotFoundError,
    handler,
    web::{Data, Path, WithContentType, WithHeader},
    IntoResponse,
};

use crate::{app_state::AppState, data::book::Book, handlers::error::HandlerError};

#[handler]
pub async fn handler(
    Path((id, format)): Path<(u64, String)>,
    state: Data<&Arc<AppState>>,
) -> Result<WithHeader<WithContentType<Vec<u8>>>, poem::Error> {
    let book = state
        .calibre
        .scalar_book(id)
        .map_err(HandlerError::DataError)?;
    let book = Book::full_book(&book, &state).ok_or(NotFoundError)?;
    let format: &str = format.as_str();
    let file_name = book.formats.get(format).ok_or(NotFoundError)?;
    let file_path = state.config.library_path.join(book.path).join(file_name);
    let mut file = File::open(file_path).map_err(|_| NotFoundError)?;

    let mut data = Vec::new();
    file.read_to_end(&mut data).map_err(|_| NotFoundError)?;
    let content_type = match format {
        "pdf" => "application/pdf",
        "epub" => "application/epub+zip",
        _ => unreachable!(),
    };

    Ok(data
        .with_content_type(content_type)
        .with_header("Content-Disposition", format!("filename={file_name};")))
}
