use calibre_db::data::error::DataStoreError;
use poem::{error::ResponseError, http::StatusCode, Body, Response};
use thiserror::Error;
use tracing::error;
use uuid::Uuid;

use crate::opds::error::OpdsError;

#[derive(Error, Debug)]
#[error("opds error")]
pub enum HandlerError {
    #[error("opds error")]
    OpdsError(#[from] OpdsError),
    #[error("data error")]
    DataError(#[from] DataStoreError),
}

impl ResponseError for HandlerError {
    fn status(&self) -> StatusCode {
        match &self {
            HandlerError::OpdsError(_) => StatusCode::INTERNAL_SERVER_ERROR,
            HandlerError::DataError(e) => match e {
                DataStoreError::NoResults(_) => StatusCode::NOT_FOUND,
                _ => StatusCode::INTERNAL_SERVER_ERROR,
            },
        }
    }

    fn as_response(&self) -> Response {
        let id = Uuid::new_v4();
        let internal_msg = format!("{:?}", self);
        let external_msg = match &self {
            HandlerError::OpdsError(_) => "internal server error",
            HandlerError::DataError(e) => match e {
                DataStoreError::NoResults(_) => "item not found",
                _ => "internal server error",
            },
        };
        error!("{id}: {internal_msg}");

        let body = Body::from_json(serde_json::json!({
            "id": id.to_string(),
            "message": external_msg,
        }))
        .unwrap();
        Response::builder().status(self.status()).body(body)
    }
}
