use std::fmt::Debug;

use calibre_db::data::error::DataStoreError;
use poem::{error::InternalServerError, web::Html};
use serde::Serialize;
use tera::Context;

use crate::templates::TEMPLATES;

use super::error::HandlerError;

pub fn render<T: Serialize + Debug, F, S, P, M>(
    template: &str,
    fetcher: F,
    sort_field: S,
    has_previous: P,
    has_more: M,
) -> Result<Html<String>, poem::Error>
where
    F: Fn() -> Result<Vec<T>, DataStoreError>,
    S: Fn(&T) -> String,
    P: Fn(&str) -> Result<bool, DataStoreError>,
    M: Fn(&str) -> Result<bool, DataStoreError>,
{
    let items = fetcher().map_err(HandlerError::DataError)?;

    let mut context = Context::new();

    // fails already in the sql query if there is nothing returned
    let first_item = items.first().unwrap();
    let last_item = items.last().unwrap();

    let (backward_cursor, forward_cursor) = (sort_field(first_item), sort_field(last_item));

    let has_previous = has_previous(&backward_cursor).map_err(HandlerError::DataError)?;
    let has_more = has_more(&forward_cursor).map_err(HandlerError::DataError)?;

    context.insert("has_previous", &has_previous);
    context.insert("has_more", &has_more);
    context.insert("backward_cursor", &backward_cursor);
    context.insert("forward_cursor", &forward_cursor);
    context.insert("nav", template);
    context.insert(template, &items);

    TEMPLATES
        .render(template, &context)
        .map_err(InternalServerError)
        .map(Html)
}
