use serde::Serialize;

#[derive(Debug, Serialize)]
#[serde(rename = "author")]
pub struct Author {
    pub name: String,
    pub uri: String,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub email: Option<String>,
}

#[cfg(test)]
mod tests {
    use quick_xml::se::to_string;

    use super::*;

    fn init() -> Author {
        Author {
            name: "Rohal der Weise".to_string(),
            uri: "https://de.wiki-aventurica.de/wiki/Rohal_der_Weise".to_string(),
            email: Some("rohal@aventurien.de".to_string()),
        }
    }

    #[test]
    fn serialize() {
        let author = init();
        let xml = to_string(&author).unwrap();
        assert_eq!(xml, "<author><name>Rohal der Weise</name><uri>https://de.wiki-aventurica.de/wiki/Rohal_der_Weise</uri><email>rohal@aventurien.de</email></author>");
    }
}
