use serde::Serialize;

use super::{media_type::MediaType, relation::Relation};

#[derive(Debug, Serialize)]
#[serde(rename = "link")]
pub struct Link {
    #[serde(rename = "@href")]
    pub href: String,
    #[serde(rename = "@type")]
    pub media_type: MediaType,
    #[serde(rename = "@rel")]
    pub rel: Relation,
    #[serde(rename = "@title")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub title: Option<String>,
    #[serde(rename = "@thr:count")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub count: Option<u64>,
}

#[cfg(test)]
mod tests {
    use quick_xml::se::to_string;

    use super::*;

    fn init() -> Link {
        Link {
            href: "/opds".to_string(),
            media_type: MediaType::Navigation,
            rel: Relation::Start,
            title: Some("Home".to_string()),
            count: Some(10),
        }
    }

    #[test]
    fn serialize() {
        let mut link = init();
        let xml = to_string(&link).unwrap();

        assert_eq!(
            xml,
            r#"<link href="/opds" type="application/atom+xml;profile=opds-catalog;kind=navigation" rel="start" title="Home" thr:count="10"/>"#
        );

        link.title = None;
        link.count = None;
        let xml = to_string(&link).unwrap();

        assert_eq!(
            xml,
            r#"<link href="/opds" type="application/atom+xml;profile=opds-catalog;kind=navigation" rel="start"/>"#
        );
    }
}
